package org.deephacks.tools4j.config.internal.core;

import org.deephacks.tools4j.config.ConfigScope;
import org.deephacks.tools4j.config.RuntimeContext;

import javax.enterprise.context.spi.Context;
import javax.enterprise.context.spi.Contextual;
import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.BeanManager;
import javax.inject.Singleton;
import java.lang.annotation.Annotation;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

@Singleton
@SuppressWarnings(value = { "unchecked", "rawtypes" })
public class ConfigCdiContext implements Context {
    private BeanManager bm;
    private RuntimeContext ctx;
    private CreationalContext cctx = null;
    private Map<Object, Object> cache = new HashMap<>();

    public ConfigCdiContext(BeanManager bm) {
        this.bm = bm;
    }

    @Override
    public Class<? extends Annotation> getScope() {
        return ConfigScope.class;
    }

    @Override
    public <T> T get(Contextual<T> contextual, CreationalContext<T> creationalContext) {
        Bean<T> bean = (Bean<T>) contextual;
        cctx = creationalContext;
        if (ctx == null) {
            ctx = lookupRuntimeContext();
        }
        return (T) ctx.singleton(bean.getBeanClass());
    }

    @Override
    public <T> T get(Contextual<T> contextual) {
        if (cctx == null) {
            return null;
        }
        if (ctx == null) {
            ctx = lookupRuntimeContext();
        }
        Bean<T> bean = (Bean<T>) contextual;
        return (T) ctx.singleton(bean.getBeanClass());
    }

    @Override
    public boolean isActive() {
        return true;
    }

    private RuntimeContext lookupRuntimeContext() {
        Set<Bean<?>> beans = bm.getBeans(RuntimeContext.class);
        Bean<?> bean = bm.resolve(beans);
        CreationalContext cc = bm.createCreationalContext(bean);
        return (RuntimeContext) bm.getReference(bean, RuntimeContext.class, cc);
    }
}